package com.gmail.tydmskz;

import java.util.ArrayList;
import java.util.List;
import java.util.Random;

import org.aiwolf.client.base.player.AbstractVillager;
import org.aiwolf.client.lib.TemplateTalkFactory;
import org.aiwolf.client.lib.Utterance;
import org.aiwolf.common.data.Agent;
import org.aiwolf.common.data.Judge;
import org.aiwolf.common.data.Role;
import org.aiwolf.common.data.Species;
import org.aiwolf.common.data.Talk;
import org.aiwolf.common.net.GameInfo;

import com.gmail.tydmskz.GameInfoGaFormatter.SpeciesEnum;

public class VillagerPlayer extends AbstractVillager{
	/*
	 * 投票アルゴリズム：人狼だと占われたエージェント，いなければ，ランダム
	 * 発話：その日に投票しようとしているエージェントを報告．変化すれば報告．
	 */

	AdvanceGameInfo agi = new AdvanceGameInfo();
	VillageInformation vi = new VillageInformation();

	//今日投票しようと思っているプレイヤー
	Agent planningVoteAgent;

	//自分が最後に宣言した「投票しようと思っているプレイヤー」
	Agent declaredPlanningVoteAgent;

	//会話をどこまで読んだか
	int readTalkListNum;

	PlayerParamaters param;

	public VillagerPlayer(PlayerParamaters p) {
		param = p;
		if(param == null || param.villagerWhoShouldIVote == null)
		{
			// throw new IllegalArgumentException("Please give me my parameters!");

			param.villagerWhoShouldIVote = new float[]{
					0.0004f, 0.0580f, -0.3886f, 0.2775f, -0.1350f, -0.2153f, -0.3342f, -0.1394f, -0.1904f, -0.1451f
			};
			
//			// てきとうにしょきか
//			float ini = 1.0f / ParametersNum;
//			for(int i=0; i<ParametersNum; i++)
//			{
//				param.villagerWhoShouldIVote[i] = ini;
//			}
		}
	}

	@Override
	public void dayStart() {
		vi.DayStart();

		declaredPlanningVoteAgent = null;
		planningVoteAgent = null;
		setPlanningVoteAgent();

		readTalkListNum =0;

	}

	@Override
	public String talk() {

		if(declaredPlanningVoteAgent != planningVoteAgent){

			String string = TemplateTalkFactory.vote(planningVoteAgent);
			declaredPlanningVoteAgent = planningVoteAgent;
			return string;
		}else{
			return TemplateTalkFactory.over();
		}
	}

	@Override
	public Agent vote() {
		return planningVoteAgent;
	}

	@Override
	public void finish() {
		// TODO 自動生成されたメソッド・スタブ

	}

	@Override
	public void update(GameInfo gameInfo) {
		super.update(gameInfo);
		vi.Update(gameInfo);

		List<Talk> talkList = gameInfo.getTalkList();
		boolean existInspectResult = false;

		/*
		 * talkListからCO，占い結果の抽出
		 */
		for(int i = readTalkListNum; i < talkList.size(); i++){
			Talk talk = talkList.get(i);
			Utterance utterance = new Utterance(talk.getContent());
			switch (utterance.getTopic()) {

			//カミングアウトの発話の場合
			case COMINGOUT:
				agi.getComingoutMap().put(talk.getAgent(), utterance.getRole());
				break;

			//占い結果の発話の場合
			case DIVINED:
				//AGIのJudgeListに結果を加える
				Agent seerAgent = talk.getAgent();
				Agent inspectedAgent = utterance.getTarget();
				Species inspectResult = utterance.getResult();
				Judge judge = new Judge(getDay(), seerAgent, inspectedAgent, inspectResult);
				agi.addInspectJudgeList(judge);

				existInspectResult =true;
				break;
			}
		}
		readTalkListNum =talkList.size();

		/*
		 * 新しい占い結果があれば投票先を変える．(新たに黒判定が出た，または投票先のプレイヤーに白判定が出た場合)
		 */
		if(existInspectResult){
			setPlanningVoteAgent();
		}
	}


	public void setPlanningVoteAgent(){
		/*
		 * 人狼だと占われたプレイヤーを指定している場合はそのまま
		 */
		if(planningVoteAgent != null){
			for(Judge judge: agi.getInspectJudgeList()){
				if(judge.getTarget().equals(planningVoteAgent)){
					return;
				}
			}
		}

		/*
		 * 投票先を未設定，または人狼だと占われたプレイヤー以外を投票先にしている場合
		 * 人狼だと占われたプレイヤーがいれば，投票先をそのプレイヤーに設定 // ←それはもしや、狂人のいいなりでは？
		 * いなければ生存プレイヤーからランダムに選択
		 */
		List<Agent> voteAgentCandidate = new ArrayList<Agent>();

		List<Agent> aliveAgentList = getLatestDayGameInfo().getAliveAgentList();
		aliveAgentList.remove(getMe());

		for(Judge judge: agi.getInspectJudgeList()){
			if(aliveAgentList.contains(judge.getTarget()) && judge.getResult() == Species.WEREWOLF){
				voteAgentCandidate.add(judge.getTarget());
			}
		}

//		// 候補がいなければ、生存プレイヤーからランダムに選択
//		if(voteAgentCandidate.size() == 0)
//		{
//			Random rand = new Random();
//			planningVoteAgent = aliveAgentList.get(rand.nextInt(aliveAgentList.size()));
//			return;
//		}

		// 候補がいれば、各候補者に対して評価値を計算
		planningVoteAgent = aliveAgentList.get(0); // 0番目のAgentで初期化
		float max = - 100000000;
		for(Agent agent:aliveAgentList)
		{
			float[] co, inspects;
			co = GameInfoGaFormatter.GetCoArray(agi, agent);
			inspects = GameInfoGaFormatter.GetInspectArray(agi, null, agent);
			
			float[] combination = new float[ParametersNum];
			System.arraycopy(co, 0, combination, 0, co.length);
			System.arraycopy(inspects, 0, combination, co.length, inspects.length);
			
			float val = Util.LinearCombine(param.villagerWhoShouldIVote, combination);
			if(max < val);
			{
				planningVoteAgent = agent;
				max = val;
			}
		}
		
		return;
	}

	public static final int ParametersNum = Role.values().length + SpeciesEnum.Count;

}
